module Test.Instances where

import Gargantext.API.Admin.EnvTypes as EnvTypes
import Gargantext.API.Admin.Auth.Types (ForgotPasswordAsyncParams(..))
import Gargantext.Core.Worker.Jobs.Types (Job(..))
import Gargantext.Prelude
import Text.Parsec.Error (ParseError, Message(..), newErrorMessage)
import Text.Parsec.Pos
import Test.Tasty.QuickCheck hiding (Positive, Negative)


instance Arbitrary EnvTypes.GargJob where
  arbitrary = do
    oneof [ pure AddAnnuaireFormJob
          , pure AddContactJob
          , pure AddCorpusFileJob
          , pure AddCorpusFormJob
          , pure AddCorpusQueryJob
          , pure AddFileJob
          , pure DocumentFromWriteNodeJob
          , pure ForgotPasswordJob
          , pure NewNodeJob
          , pure RecomputeGraphJob
          , pure TableNgramsJob
          , pure UpdateNgramsListJobJSON
          , pure UpdateNgramsListJobTSV
          , pure UpdateNodeJob
          , pure UploadDocumentJob
          , pure UploadFrameCalcJob
          ]


instance Arbitrary Job where
  arbitrary = oneof [ pure Ping
                    , forgotPasswordAsyncGen
                    , gargJobGen ]
    where
      forgotPasswordAsyncGen = do
        email <- arbitrary
        return $ ForgotPasswordAsync (ForgotPasswordAsyncParams { email })
      gargJobGen = do
        _gj_garg_job <- arbitrary
        return $ GargJob { _gj_garg_job }




instance Arbitrary Message where
  arbitrary = do
    msgContent <- arbitrary
    oneof $ return <$> [SysUnExpect msgContent
                       , UnExpect msgContent
                       , Expect msgContent
                       , Message msgContent
                       ]

instance Arbitrary SourcePos where
  arbitrary = do
    sn <- arbitrary
    l <- arbitrary
    c <- arbitrary
    return $ newPos sn l c

instance Arbitrary ParseError where
  arbitrary = do
    sp <- arbitrary
    msg <- arbitrary
    return $ newErrorMessage msg sp
